@@ -8,8 +8,8 @@
 #include "config.h"
 
 #include "adw-indicator-bin-private.h"
+#include "adw-macros-private.h"
 #include "adw-view-switcher-button-private.h"
-#include "adw-widget-utils-private.h"
 
 #define TIMEOUT_EXPAND 500
 
@@ -55,12 +55,14 @@ static GParamSpec *props[LAST_PROP];
 G_DEFINE_FINAL_TYPE_WITH_CODE (AdwViewSwitcherButton, adw_view_switcher_button, GTK_TYPE_TOGGLE_BUTTON,
                                G_IMPLEMENT_INTERFACE (GTK_TYPE_ORIENTABLE, NULL))
 
-static void
-switch_timeout_cb (AdwViewSwitcherButton *self)
+static gboolean
+adw_view_switcher_button_switch_timeout (AdwViewSwitcherButton *self)
 {
   self->switch_timer = 0;
 
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (self), TRUE);
+
+  return G_SOURCE_REMOVE;
 }
 
 static void
@@ -82,11 +84,10 @@ drag_enter_cb (AdwViewSwitcherButton *self)
   if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (self)))
     return;
 
-  self->switch_timer =
-    g_timeout_add_once (TIMEOUT_EXPAND,
-                        (GSourceOnceFunc) switch_timeout_cb,
-                        self);
-  g_source_set_name_by_id (self->switch_timer, "[adw] switch_timeout_cb");
+  self->switch_timer = g_timeout_add (TIMEOUT_EXPAND,
+                                      (GSourceFunc) adw_view_switcher_button_switch_timeout,
+                                      self);
+  g_source_set_name_by_id (self->switch_timer, "[gtk] adw_view_switcher_switch_timeout");
 }
 
 static void
@@ -126,7 +127,13 @@ static gchar *
 get_badge_text (AdwViewSwitcherButton *self,
                 guint                  badge_number)
 {
-  return adw_get_badge_text (badge_number);
+  if (badge_number > 999)
+    return g_strdup ("999+");
+
+  if (!badge_number)
+    return g_strdup ("");
+
+  return g_strdup_printf ("%u", badge_number);
 }
 
 static void
@@ -267,9 +274,11 @@ adw_view_switcher_button_class_init (AdwViewSwitcherButtonClass *klass)
                                     "orientation");
 
   /**
-   * AdwViewSwitcherButton:icon-name:
+   * AdwViewSwitcherButton:icon-name: (attributes org.gtk.Property.get=adw_view_switcher_button_get_icon_name org.gtk.Property.set=adw_view_switcher_button_set_icon_name)
    *
    * The icon name representing the view, or `NULL` for no icon.
+   *
+   * Since: 1.0
    */
   props[PROP_ICON_NAME] =
     g_param_spec_string ("icon-name", NULL, NULL,
@@ -277,13 +286,15 @@ adw_view_switcher_button_class_init (AdwViewSwitcherButtonClass *klass)
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwViewSwitcherButton:needs-attention:
+   * AdwViewSwitcherButton:needs-attention: (attributes org.gtk.Property.get=adw_view_switcher_button_get_needs_attention org.gtk.Property.set=adw_view_switcher_button_set_needs_attention)
    *
    * Sets a flag specifying whether the view requires the user attention.
    *
    * This is used by the [class@ViewSwitcher] to change the appearance of the
    * corresponding button when a view needs attention and it is not the current
    * one.
+   *
+   * Since: 1.0
    */
   props[PROP_NEEDS_ATTENTION] =
     g_param_spec_boolean ("needs-attention", NULL, NULL,
@@ -294,6 +305,8 @@ adw_view_switcher_button_class_init (AdwViewSwitcherButtonClass *klass)
    * AdwViewSwitcherButton:badge-number:
    *
    * A number to display as a badge on the button.
+   *
+   * Since: 1.0
    */
   props[PROP_BADGE_NUMBER] =
     g_param_spec_uint ("badge-number", NULL, NULL,
@@ -350,6 +363,8 @@ adw_view_switcher_button_init (AdwViewSwitcherButton *self)
  * Creates a new `AdwViewSwitcherButton`.
  *
  * Returns: the newly created `AdwViewSwitcherButton`
+ *
+ * Since: 1.0
  */
 GtkWidget *
 adw_view_switcher_button_new (void)
@@ -358,12 +373,14 @@ adw_view_switcher_button_new (void)
 }
 
 /**
- * adw_view_switcher_button_get_icon_name:
+ * adw_view_switcher_button_get_icon_name: (attributes org.gtk.Method.get_property=icon-name)
  * @self: a view switcher button
  *
  * Gets the icon name representing the view.
  *
  * Returns: (nullable): the icon name
+ *
+ * Since: 1.0
  **/
 const char *
 adw_view_switcher_button_get_icon_name (AdwViewSwitcherButton *self)
@@ -374,11 +391,13 @@ adw_view_switcher_button_get_icon_name (AdwViewSwitcherButton *self)
 }
 
 /**
- * adw_view_switcher_button_set_icon_name:
+ * adw_view_switcher_button_set_icon_name: (attributes org.gtk.Method.set_property=icon-name)
  * @self: a view switcher button
  * @icon_name: (nullable): an icon name
  *
  * Sets the icon name representing the view.
+ *
+ * Since: 1.0
  **/
 void
 adw_view_switcher_button_set_icon_name (AdwViewSwitcherButton *self,
@@ -389,21 +408,25 @@ adw_view_switcher_button_set_icon_name (AdwViewSwitcherButton *self,
   if (!g_strcmp0 (self->icon_name, icon_name))
     return;
 
-  if (icon_name && *icon_name)
-    g_set_str (&self->icon_name, icon_name);
+  g_free (self->icon_name);
+
+  if (icon_name && strlen (icon_name) > 0)
+    self->icon_name = g_strdup (icon_name);
   else
-    g_set_str (&self->icon_name, "image-missing");
+    self->icon_name = g_strdup ("image-missing");
 
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_ICON_NAME]);
 }
 
 /**
- * adw_view_switcher_button_get_needs_attention:
+ * adw_view_switcher_button_get_needs_attention: (attributes org.gtk.Method.get_property=needs-attention)
  * @self: a view switcher button
  *
  * Gets whether the view represented by @self requires the user attention.
  *
  * Returns: whether the view requires the user attention
+ *
+ * Since: 1.0
  **/
 gboolean
 adw_view_switcher_button_get_needs_attention (AdwViewSwitcherButton *self)
@@ -414,11 +437,13 @@ adw_view_switcher_button_get_needs_attention (AdwViewSwitcherButton *self)
 }
 
 /**
- * adw_view_switcher_button_set_needs_attention:
+ * adw_view_switcher_button_set_needs_attention: (attributes org.gtk.Method.set_property=needs-attention)
  * @self: a view switcher button
  * @needs_attention: whether the view needs attention
  *
  * Sets whether the view represented by @self requires the user attention.
+ *
+ * Since: 1.0
  */
 void
 adw_view_switcher_button_set_needs_attention (AdwViewSwitcherButton *self,
@@ -443,6 +468,8 @@ adw_view_switcher_button_set_needs_attention (AdwViewSwitcherButton *self,
  * Gets the badge number.
  *
  * Returns: the badge number
+ *
+ * Since: 1.0
  */
 guint
 adw_view_switcher_button_get_badge_number (AdwViewSwitcherButton *self)
@@ -458,6 +485,8 @@ adw_view_switcher_button_get_badge_number (AdwViewSwitcherButton *self)
  * @badge_number: the new value
  *
  * Sets the badge number.
+ *
+ * Since: 1.0
  */
 void
 adw_view_switcher_button_set_badge_number (AdwViewSwitcherButton *self,
@@ -480,6 +509,8 @@ adw_view_switcher_button_set_badge_number (AdwViewSwitcherButton *self,
  * Gets the label representing the view.
  *
  * Returns: (nullable): the label
+ *
+ * Since: 1.0
  **/
 const char *
 adw_view_switcher_button_get_label (AdwViewSwitcherButton *self)
@@ -495,6 +526,8 @@ adw_view_switcher_button_get_label (AdwViewSwitcherButton *self)
  * @label: (nullable): a label
  *
  * Sets the label representing the view.
+ *
+ * Since: 1.0
  **/
 void
 adw_view_switcher_button_set_label (AdwViewSwitcherButton *self,
@@ -502,8 +535,11 @@ adw_view_switcher_button_set_label (AdwViewSwitcherButton *self,
 {
   g_return_if_fail (ADW_IS_VIEW_SWITCHER_BUTTON (self));
 
-  if (!g_set_str (&self->label, label))
+  if (!g_strcmp0 (self->label, label))
     return;
 
+  g_free (self->label);
+  self->label = g_strdup (label);
+
   g_object_notify (G_OBJECT (self), "label");
 }
